## Internationalization

The default locale of VueCesium is Chinese. If you want to use another language, you'll need to do some i18n configuration. In your entry file, if you are importing VueCesium entirely:

```javascript
import { createApp } from 'vue'
import VueCesium from 'vue-cesium'
import locale from 'vue-cesium/lib/locale/lang/en-us'

createApp(App).use(VueCesium, { locale })
```

if you are importing VueCesium on demand via es modules:

```typescript
import { createApp } from 'vue'
import { Viewer, locale } from 'vue-cesium'
import lang from 'vue-cesium/locale/lang/en-us'

locale(lang)
createApp(App).component(Viewer.name, Viewer)
```

Or if you are importing VueCesium on demand via [babel-plugin-component](#/en-US/component/quickstart#on-demand):

```typescript
import { createApp } from 'vue'
import { Viewer } from 'vue-cesium'
import lang from 'vue-cesium/lib/locale/lang/en-us'
import locale from 'vue-cesium/lib/locale'

// configure language
locale.use(lang)

// import components
createApp(App).component(Viewer.name, Viewer)
```


The English locale is imported by default, even if you're using another locale. But with NormalModuleReplacementPlugin provided by webpack you can replace default locale:

**webpack.config.js**

```typescript
{
  plugins: [
    new webpack.NormalModuleReplacementPlugin(
      /vue-cesium[\/\\]lib[\/\\]locale[\/\\]lang[\/\\]zh-Hans/,
      'vue-cesium/lib/locale/lang/en-us',
    ),
  ]
}
```


### Compatible `vue-i18n@9.x`

If you need to check out [VueI18n documentation](https://vue-i18n-next.intlify.dev/guide/#html), please click this link to check it out.

```typescript
import { createApp } from 'vue'
import { createI18n } from 'vue-i18n'
import VueCesium from 'vue-cesium'
import enLocale from 'vue-cesium/lib/locale/lang/en-us'
import zhLocale from 'vue-cesium/lib/locale/lang/zh-Hans'
import App from './App.vue'

const messages = {
  [enLocale.name]: {
    // vc property is critical, set this in order for VueCesium translate strings correctly.
    vc: enLocale.vc,
    // Define your own dictionary with your own namespace, but DO NOT use the namespace `vc`.
    // Because that makes the VueCesium internal translation invalid.
    message: {
      hello: 'hello world',
    },
  },
  [zhLocale.name]: {
    vc: zhLocale.vc,
    // Define your own dictionary with your own namespace, but DO NOT use the namespace `vc`,
    // Because that makes the VueCesium internal translation invalid.
    message: {
      hello: '你好，世界',
    },
  },
  testLocale: {
    vc: {},
    // No message translations, it will fallback to en lang, the definition of fallbackLocale is below 👇
  },
}

const i18n = createI18n({
  locale: zhLocale.name,
  fallbackLocale: enLocale.name,
  messages,
})

const app = createApp(App)

app.use(VueCesium, {
  i18n: i18n.global.t,
})

// Remember to use this plugin.
app.use(i18n)
```

Currently VueCesium ships with the following languages:

<ul class="language-list">
  <li>Simplified Chinese(zh-hans)</li>
  <li>English(en-us)</li>
</ul>

If your target language is not included, you are more than welcome to contribute: just add another language config [here](https://github.com/zouyaoji/vue-cesium/tree/dev/packages/locale/lang) and create a pull request.
